@@ -12,6 +12,20 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-from pallas.assembly import environ_setup, setup
+from __future__ import annotations
 
-__all__ = ["environ_setup", "setup"]
+from pallas.assembly import environ_setup, setup
+from pallas.client import Athena, Query
+from pallas.exceptions import AthenaQueryError
+from pallas.info import QueryInfo
+from pallas.results import QueryResults
+
+__all__ = [
+    "environ_setup",
+    "setup",
+    "Athena",
+    "Query",
+    "QueryInfo",
+    "QueryResults",
+    "AthenaQueryError",
+]
