@@ -13,27 +13,28 @@
 # limitations under the License.
 
 """
-Fake Athena client for testing purposes.
+Helpers for testing.
 """
 
 from __future__ import annotations
 
 from typing import Dict, List, Optional, Sequence
 
-from pallas.base import Athena, Query
 from pallas.info import QueryInfo
+from pallas.proxies import AthenaProxy
 from pallas.results import QueryResults
 
 
-class AthenaFake(Athena):
-    """Fake Athena implementation that can be used for testing."""
+class FakeProxy(AthenaProxy):
+    """
+    Fake of proxy to AWS Athena.
 
-    database: Optional[str] = None
-    workgroup: Optional[str] = None
-    output_location: Optional[str] = None
+    Can replace :class:`.Boto3Proxy` for testing.
+    """
 
     column_names: Optional[Sequence[str]] = None
     column_types: Optional[Sequence[str]] = None
+    state = "SUCCEEDED"
     data: Optional[Sequence[Sequence[str]]] = None
 
     _sql: Dict[str, str]
@@ -49,27 +50,40 @@ class AthenaFake(Athena):
     def request_log(self) -> List[str]:
         return self._request_log
 
-    def submit(self, sql: str, *, ignore_cache: bool = False) -> Query:
+    def start_query_execution(
+        self,
+        sql: str,
+        database: Optional[str] = None,
+        workgroup: Optional[str] = None,
+        output_location: Optional[str] = None,
+    ) -> str:
         execution_id = f"query-{len(self._results) + 1}"
         self._request_log.append("StartQueryExecution")
-        results = self._get_results()
+        results = self._fake_query_results()
         self._sql[execution_id] = sql
         self._results[execution_id] = results
-        return self.get_query(execution_id)
+        return execution_id
 
-    def get_query(self, execution_id: str) -> Query:
-        info = self._get_info(execution_id, self._sql[execution_id])
-        results = self._results[execution_id]
-        return QueryFake(info, results, self._request_log)
+    def get_query_execution(self, execution_id: str) -> QueryInfo:
+        self._request_log.append("GetQueryExecution")
+        return self._fake_query_info(execution_id, self._sql[execution_id])
 
-    def _get_info(self, execution_id: str, sql: str) -> QueryInfo:
+    def get_query_results(self, info: QueryInfo) -> QueryResults:
+        self._request_log.append("GetQueryResults")
+        return self._results[info.execution_id]
+
+    def stop_query_execution(self, execution_id: str) -> None:
+        self._request_log.append("StopQueryExecution")
+        self.state = "CANCELLED"
+
+    def _fake_query_info(self, execution_id: str, sql: str) -> QueryInfo:
         data = {
             "QueryExecutionId": execution_id,
             "Query": sql,
             "ResultConfiguration": {"OutputLocation": ...},
             "QueryExecutionContext": {},
             "Status": {
-                "State": "SUCCEEDED",
+                "State": self.state,
                 "SubmissionDateTime": ...,
                 "CompletionDateTime": ...,
             },
@@ -84,7 +98,7 @@ class AthenaFake(Athena):
         }
         return QueryInfo(data)
 
-    def _get_results(self) -> QueryResults:
+    def _fake_query_results(self) -> QueryResults:
         column_names = self.column_names
         column_types = self.column_types
         data = self.data
@@ -95,44 +109,3 @@ class AthenaFake(Athena):
         if data is None:
             data = []
         return QueryResults(column_names, column_types, data)
-
-
-class QueryFake(Query):
-    """Fake Query implementation that can be used for testing."""
-
-    _info: QueryInfo
-    _results: QueryResults
-    _request_log: List[str]
-
-    _finished_info: Optional[QueryInfo] = None
-
-    def __init__(
-        self, info: QueryInfo, results: QueryResults, request_log: List[str]
-    ) -> None:
-        self._info = info
-        self._results = results
-        self._request_log = request_log
-
-    @property
-    def execution_id(self) -> str:
-        return self._info.execution_id
-
-    def get_info(self) -> QueryInfo:
-        # Mimic behavior of QueryProxy - remember info of finished queries.
-        if self._finished_info is not None:
-            return self._finished_info
-        self._request_log.append("GetQueryExecution")
-        if self._info.finished:
-            self._finished_info = self._info
-        return self._info
-
-    def get_results(self) -> QueryResults:
-        self.join()
-        self._request_log.append("GetQueryResults")
-        return self._results
-
-    def kill(self) -> None:
-        self._request_log.append("StopQueryExecution")
-
-    def join(self) -> None:
-        self.get_info().check()
